#include <gtk/gtk.h>

#include <pango/pangocairo.h>


PangoContext* pango_context;
PangoLayout* pango_layout;
PangoFontDescription* font_description;
GtkWidget *drawing_area;
gint zoom_level = 1;

/* Draw the text */
gboolean
expose_event_callback (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
    cairo_t *cr;
    cairo_matrix_t matrix;


    cr = gdk_cairo_create (widget->window);

    cairo_matrix_init_identity (&matrix);
    cairo_matrix_scale (&matrix, zoom_level, zoom_level);
    matrix.x0 = 10;
    matrix.y0 = 10;
    cairo_transform (cr, &matrix);
    pango_cairo_update_layout (cr, pango_layout);

    pango_cairo_show_layout (cr, pango_layout);

    cairo_destroy (cr);
 
    return TRUE;
}

/* Zoom in */
void
zoom_in_callback (GtkButton *button, gpointer user_data)
{
    zoom_level += 1;
    gtk_widget_queue_draw (drawing_area);
}

/* Zoom out */
void
zoom_out_callback (GtkButton *button, gpointer user_data)
{
    if (zoom_level >= 2) {
        zoom_level -= 1;
        gtk_widget_queue_draw (drawing_area);
    }
}

int
main (int argc, char **argv)
{
    GtkWidget *window;
    GtkWidget* vbox;
    GtkWidget* buttons_box;
    GtkWidget* zoom_in;
    GtkWidget* zoom_out;

    gtk_init (&argc, &argv);

    /* Window construction (buttons, containers, etc) */

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);

    drawing_area = gtk_drawing_area_new ();
    gtk_container_add (GTK_CONTAINER (vbox), drawing_area);
    gtk_widget_set_size_request (drawing_area, 500, 300);

    buttons_box = gtk_hbox_new (TRUE, 2);
    gtk_container_add (GTK_CONTAINER (vbox), buttons_box);
    gtk_widget_set_size_request (buttons_box, 500, 30);

    zoom_in = gtk_button_new_with_label ("+");
    gtk_container_add (GTK_CONTAINER (buttons_box), zoom_in);

    zoom_out = gtk_button_new_with_label ("-");
    gtk_container_add (GTK_CONTAINER (buttons_box), zoom_out);

    /* Connecting signals */

    g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);

    g_signal_connect (G_OBJECT (drawing_area), "expose_event",  
                    G_CALLBACK (expose_event_callback), NULL);

    g_signal_connect (zoom_in, "clicked", G_CALLBACK (zoom_in_callback), NULL);
    g_signal_connect (zoom_out, "clicked", G_CALLBACK (zoom_out_callback), NULL);

    /* Initialization of Pango stuff */

    font_description = pango_font_description_new ();
    pango_font_description_set_family (font_description, "Times New Roman");
    pango_font_description_set_size (font_description, 6*PANGO_SCALE);

    pango_context = pango_cairo_font_map_create_context ( PANGO_CAIRO_FONT_MAP (pango_cairo_font_map_get_default ()) );

    pango_layout = pango_layout_new (pango_context);
    pango_layout_set_text (pango_layout, "Hello World!", -1);
    pango_layout_set_font_description (pango_layout, font_description);

    gtk_widget_show_all (window);

    gtk_main ();

    pango_font_description_free(font_description);
    g_object_unref(pango_layout);
    g_object_unref(pango_context);
}
